@@ -16,14 +16,20 @@ TODO:
 __all__ = ('WindowSDL2', )
 
 from os.path import join
+import sys
 from kivy import kivy_data_dir
 from kivy.logger import Logger
-from kivy import metrics
 from kivy.base import EventLoop, ExceptionManager, stopTouchApp
 from kivy.clock import Clock
 from kivy.config import Config
 from kivy.core.window import WindowBase
-from kivy.core.window._window_sdl2 import _WindowSDL2Storage
+try:
+    from kivy.core.window._window_sdl2 import _WindowSDL2Storage
+except ImportError:
+    from kivy.core import handle_win_lib_import_error
+    handle_win_lib_import_error(
+        'window', 'sdl2', 'kivy.core.window._window_sdl2')
+    raise
 from kivy.input.provider import MotionEventProvider
 from kivy.input.motionevent import MotionEvent
 from kivy.resources import resource_find
@@ -31,14 +37,20 @@ from kivy.utils import platform, deprecated
 from kivy.compat import unichr
 from collections import deque
 
-KMOD_LCTRL = 64
-KMOD_RCTRL = 128
-KMOD_RSHIFT = 2
-KMOD_LSHIFT = 1
-KMOD_RALT = 512
-KMOD_LALT = 256
-KMOD_LMETA = 1024
-KMOD_RMETA = 2048
+
+# SDL_keycode.h, https://wiki.libsdl.org/SDL_Keymod
+KMOD_NONE = 0x0000
+KMOD_LSHIFT = 0x0001
+KMOD_RSHIFT = 0x0002
+KMOD_LCTRL = 0x0040
+KMOD_RCTRL = 0x0080
+KMOD_LALT = 0x0100
+KMOD_RALT = 0x0200
+KMOD_LGUI = 0x0400
+KMOD_RGUI = 0x0800
+KMOD_NUM = 0x1000
+KMOD_CAPS = 0x2000
+KMOD_MODE = 0x4000
 
 SDLK_SHIFTL = 1073742049
 SDLK_SHIFTR = 1073742053
@@ -58,6 +70,22 @@ SDLK_SUPER = 1073742051
 SDLK_CAPS = 1073741881
 SDLK_INSERT = 1073741897
 SDLK_KEYPADNUM = 1073741907
+SDLK_KP_DEVIDE = 1073741908
+SDLK_KP_MULTIPLY = 1073741909
+SDLK_KP_MINUS = 1073741910
+SDLK_KP_PLUS = 1073741911
+SDLK_KP_ENTER = 1073741912
+SDLK_KP_1 = 1073741913
+SDLK_KP_2 = 1073741914
+SDLK_KP_3 = 1073741915
+SDLK_KP_4 = 1073741916
+SDLK_KP_5 = 1073741917
+SDLK_KP_6 = 1073741918
+SDLK_KP_7 = 1073741919
+SDLK_KP_8 = 1073741920
+SDLK_KP_9 = 1073741921
+SDLK_KP_0 = 1073741922
+SDLK_KP_DOT = 1073741923
 SDLK_F1 = 1073741882
 SDLK_F2 = 1073741883
 SDLK_F3 = 1073741884
@@ -116,13 +144,17 @@ class SDL2MotionEventProvider(MotionEventProvider):
 
 class WindowSDL(WindowBase):
 
+    _do_resize_ev = None
+
     def __init__(self, **kwargs):
+        self._pause_loop = False
         self._win = _WindowSDL2Storage()
         super(WindowSDL, self).__init__()
         self._mouse_x = self._mouse_y = -1
-        self._meta_keys = (KMOD_LCTRL, KMOD_RCTRL, KMOD_RSHIFT,
-            KMOD_LSHIFT, KMOD_RALT, KMOD_LALT, KMOD_LMETA,
-            KMOD_RMETA)
+        self._meta_keys = (
+            KMOD_LCTRL, KMOD_RCTRL, KMOD_RSHIFT,
+            KMOD_LSHIFT, KMOD_RALT, KMOD_LALT, KMOD_LGUI,
+            KMOD_RGUI, KMOD_NUM, KMOD_CAPS, KMOD_MODE)
         self.command_keys = {
                     27: 'escape',
                     9: 'tab',
@@ -139,29 +171,115 @@ class WindowSDL(WindowBase):
                     280: 'pgup',
                     281: 'pgdown'}
         self._mouse_buttons_down = set()
+        self.key_map = {SDLK_LEFT: 276, SDLK_RIGHT: 275, SDLK_UP: 273,
+                        SDLK_DOWN: 274, SDLK_HOME: 278, SDLK_END: 279,
+                        SDLK_PAGEDOWN: 281, SDLK_PAGEUP: 280, SDLK_SHIFTR: 303,
+                        SDLK_SHIFTL: 304, SDLK_SUPER: 309, SDLK_LCTRL: 305,
+                        SDLK_RCTRL: 306, SDLK_LALT: 308, SDLK_RALT: 307,
+                        SDLK_CAPS: 301, SDLK_INSERT: 277, SDLK_F1: 282,
+                        SDLK_F2: 283, SDLK_F3: 284, SDLK_F4: 285, SDLK_F5: 286,
+                        SDLK_F6: 287, SDLK_F7: 288, SDLK_F8: 289, SDLK_F9: 290,
+                        SDLK_F10: 291, SDLK_F11: 292, SDLK_F12: 293,
+                        SDLK_F13: 294, SDLK_F14: 295, SDLK_F15: 296,
+                        SDLK_KEYPADNUM: 300, SDLK_KP_DEVIDE: 267,
+                        SDLK_KP_MULTIPLY: 268, SDLK_KP_MINUS: 269,
+                        SDLK_KP_PLUS: 270, SDLK_KP_ENTER: 271,
+                        SDLK_KP_DOT: 266, SDLK_KP_0: 256, SDLK_KP_1: 257,
+                        SDLK_KP_2: 258, SDLK_KP_3: 259, SDLK_KP_4: 260,
+                        SDLK_KP_5: 261, SDLK_KP_6: 262, SDLK_KP_7: 263,
+                        SDLK_KP_8: 264, SDLK_KP_9: 265}
+        if platform == 'ios':
+            # XXX ios keyboard suck, when backspace is hit, the delete
+            # keycode is sent. fix it.
+            self.key_map[127] = 8
+        elif platform == 'android':
+            # map android back button to escape
+            self.key_map[1073742094] = 27
+
+        self.bind(minimum_width=self._set_minimum_size,
+                  minimum_height=self._set_minimum_size)
+
+        self.bind(allow_screensaver=self._set_allow_screensaver)
+
+    def get_window_info(self):
+        return self._win.get_window_info()
+
+    def _set_minimum_size(self, *args):
+        minimum_width = self.minimum_width
+        minimum_height = self.minimum_height
+        if minimum_width and minimum_height:
+            self._win.set_minimum_size(minimum_width, minimum_height)
+        elif minimum_width or minimum_height:
+            Logger.warning(
+                'Both Window.minimum_width and Window.minimum_height must be '
+                'bigger than 0 for the size restriction to take effect.')
+
+    def _set_allow_screensaver(self, *args):
+        self._win.set_allow_screensaver(self.allow_screensaver)
+
+    def _event_filter(self, action, *largs):
+        from kivy.app import App
+        if action == 'app_terminating':
+            EventLoop.quit = True
+
+        elif action == 'app_lowmemory':
+            self.dispatch('on_memorywarning')
+
+        elif action == 'app_willenterbackground':
+            from kivy.base import stopTouchApp
+            app = App.get_running_app()
+            if not app:
+                Logger.info('WindowSDL: No running App found, exit.')
+                stopTouchApp()
+                return 0
+
+            if not app.dispatch('on_pause'):
+                Logger.info(
+                    'WindowSDL: App doesn\'t support pause mode, stop.')
+                stopTouchApp()
+                return 0
+
+            self._pause_loop = True
+
+        elif action == 'app_didenterforeground':
+            # on iOS, the did enter foreground is launched at the start
+            # of the application. in our case, we want it only when the app
+            # is resumed
+            if self._pause_loop:
+                self._pause_loop = False
+                app = App.get_running_app()
+                app.dispatch('on_resume')
+
+        elif action == 'windowresized':
+            self._size = largs
+            self._win.resize_window(*self._size)
+            # Force kivy to render the frame now, so that the canvas is drawn.
+            EventLoop.idle()
+
+        return 0
 
     def create_window(self, *largs):
-
         if self._fake_fullscreen:
             if not self.borderless:
                 self.fullscreen = self._fake_fullscreen = False
             elif not self.fullscreen or self.fullscreen == 'auto':
                 self.borderless = self._fake_fullscreen = False
-
         if self.fullscreen == 'fake':
             self.borderless = self._fake_fullscreen = True
             Logger.warning("The 'fake' fullscreen option has been "
-                            "deprecated, use Window.borderless or the "
-                            "borderless Config option instead.")
+                           "deprecated, use Window.borderless or the "
+                           "borderless Config option instead.")
 
         if not self.initialized:
-
             if self.position == 'auto':
                 pos = None, None
             elif self.position == 'custom':
                 pos = self.left, self.top
 
-            # setup !
+            # ensure we have an event filter
+            self._win.set_event_filter(self._event_filter)
+
+            # setup window
             w, h = self.system_size
             resizable = Config.getboolean('graphics', 'resizable')
             state = (Config.get('graphics', 'window_state')
@@ -179,6 +297,11 @@ class WindowSDL(WindowBase):
             # never stay with a None pos, application using w.center
             # will be fired.
             self._pos = (0, 0)
+            self._set_minimum_size()
+            self._set_allow_screensaver()
+
+            if state == 'hidden':
+                self._focus = False
         else:
             w, h = self.system_size
             self._win.resize_window(w, h)
@@ -186,6 +309,8 @@ class WindowSDL(WindowBase):
             self._win.set_fullscreen_mode(self.fullscreen)
 
         super(WindowSDL, self).create_window()
+        # set mouse visibility
+        self._set_cursor_state(self.show_cursor)
 
         if self.initialized:
             return
@@ -214,7 +339,8 @@ class WindowSDL(WindowBase):
 
     def close(self):
         self._win.teardown_window()
-        self.dispatch('on_close')
+        super(WindowSDL, self).close()
+        self.initialized = False
 
     def maximize(self):
         if self._is_desktop:
@@ -246,6 +372,12 @@ class WindowSDL(WindowBase):
         else:
             Logger.warning('Window: show() is used only on desktop OSes.')
 
+    def raise_window(self):
+        if self._is_desktop:
+            self._win.raise_window()
+        else:
+            Logger.warning('Window: show() is used only on desktop OSes.')
+
     @deprecated
     def toggle_fullscreen(self):
         if self.fullscreen in (True, 'auto'):
@@ -275,20 +407,95 @@ class WindowSDL(WindowBase):
         self._win.flip()
         super(WindowSDL, self).flip()
 
+    def set_system_cursor(self, cursor_name):
+        result = self._win.set_system_cursor(cursor_name)
+        return result
+
+    def _get_window_pos(self):
+        return self._win.get_window_pos()
+
+    def _set_window_pos(self, x, y):
+        self._win.set_window_pos(x, y)
+
+    # Transparent Window background
+    def _is_shaped(self):
+        return self._win.is_window_shaped()
+
+    def _set_shape(self, shape_image, mode='default',
+                   cutoff=False, color_key=None):
+        modes = ('default', 'binalpha', 'reversebinalpha', 'colorkey')
+        color_key = color_key or (0, 0, 0, 1)
+        if mode not in modes:
+            Logger.warning(
+                'Window: shape mode can be only '
+                '{}'.format(', '.join(modes))
+            )
+            return
+        if not isinstance(color_key, (tuple, list)):
+            return
+        if len(color_key) not in (3, 4):
+            return
+        if len(color_key) == 3:
+            color_key = (color_key[0], color_key[1], color_key[2], 1)
+            Logger.warning(
+                'Window: Shape color_key must be only tuple or list'
+            )
+            return
+        color_key = (
+            color_key[0] * 255,
+            color_key[1] * 255,
+            color_key[2] * 255,
+            color_key[3] * 255
+        )
+
+        assert cutoff in (1, 0)
+        shape_image = shape_image or Config.get('kivy', 'window_shape')
+        shape_image = resource_find(shape_image) or shape_image
+        self._win.set_shape(shape_image, mode, cutoff, color_key)
+
+    def _get_shaped_mode(self):
+        return self._win.get_shaped_mode()
+
+    def _set_shaped_mode(self, value):
+        self._set_shape(
+            shape_image=self.shape_image,
+            mode=value, cutoff=self.shape_cutoff,
+            color_key=self.shape_color_key
+        )
+        return self._win.get_shaped_mode()
+    # twb end
+
+    def _set_cursor_state(self, value):
+        self._win._set_cursor_state(value)
+
     def _fix_mouse_pos(self, x, y):
-        density = self._density
         y -= 1
-        if self._is_desktop and self.size[0] != self._size[0]:
-            x, y = x * density, (y * density) - self.system_size[1]
-            #y = self.system_size[1] - y
-            self.mouse_pos = x, y
-        else:
-            self.mouse_pos = x, self.system_size[1] - y
+        self.mouse_pos = (x * self._density,
+                          (self.system_size[1] - y) * self._density)
         return x, y
 
     def _mainloop(self):
         EventLoop.idle()
 
+        # for android/iOS, we don't want to have any event nor executing our
+        # main loop while the pause is going on. This loop wait any event (not
+        # handled by the event filter), and remove them from the queue.
+        # Nothing happen during the pause on iOS, except gyroscope value sent
+        # over joystick. So it's safe.
+        while self._pause_loop:
+            self._win.wait_event()
+            if not self._pause_loop:
+                break
+            event = self._win.poll()
+            if event is None:
+                continue
+            # As dropfile is send was the app is still in pause.loop
+            # we need to dispatch it
+            action, args = event[0], event[1:]
+            if action == 'dropfile':
+                dropfile = args
+                self.dispatch('on_dropfile', dropfile[0])
+
         while True:
             event = self._win.poll()
             if event is False:
@@ -298,8 +505,9 @@ class WindowSDL(WindowBase):
 
             action, args = event[0], event[1:]
             if action == 'quit':
+                if self.dispatch('on_request_close'):
+                    continue
                 EventLoop.quit = True
-                self.close()
                 break
 
             elif action in ('fingermotion', 'fingerdown', 'fingerup'):
@@ -309,7 +517,7 @@ class WindowSDL(WindowBase):
                 # We have a conflict of using either the mouse or the finger.
                 # Right now, we have no mechanism that we could use to know
                 # which is the preferred one for the application.
-                if platform == "ios":
+                if platform in ('ios', 'android'):
                     SDL2MotionEventProvider.q.appendleft(event)
                 pass
 
@@ -353,9 +561,9 @@ class WindowSDL(WindowBase):
 
                 self._mouse_meta = self.modifiers
                 self._mouse_btn = btn
-                #times = x if y == 0 else y
-                #times = min(abs(times), 100)
-                #for k in range(times):
+                # times = x if y == 0 else y
+                # times = min(abs(times), 100)
+                # for k in range(times):
                 self._mouse_down = True
                 self.dispatch('on_mouse_down',
                     self._mouse_x, self._mouse_y, btn, self.modifiers)
@@ -370,23 +578,49 @@ class WindowSDL(WindowBase):
             elif action == 'windowresized':
                 self._size = self._win.window_size
                 # don't use trigger here, we want to delay the resize event
-                cb = self._do_resize
-                Clock.unschedule(cb)
-                Clock.schedule_once(cb, .1)
+                ev = self._do_resize_ev
+                if ev is None:
+                    ev = Clock.schedule_once(self._do_resize, .1)
+                    self._do_resize_ev = ev
+                else:
+                    ev()
 
-            elif action == 'windowresized':
-                self.canvas.ask_update()
+            elif action == 'windowmoved':
+                self.dispatch('on_move')
 
             elif action == 'windowrestored':
+                self.dispatch('on_restore')
                 self.canvas.ask_update()
 
             elif action == 'windowexposed':
                 self.canvas.ask_update()
 
             elif action == 'windowminimized':
+                self.dispatch('on_minimize')
                 if Config.getboolean('kivy', 'pause_on_minimize'):
                     self.do_pause()
 
+            elif action == 'windowmaximized':
+                self.dispatch('on_maximize')
+
+            elif action == 'windowhidden':
+                self.dispatch('on_hide')
+
+            elif action == 'windowshown':
+                self.dispatch('on_show')
+
+            elif action == 'windowfocusgained':
+                self._focus = True
+
+            elif action == 'windowfocuslost':
+                self._focus = False
+
+            elif action == 'windowenter':
+                self.dispatch('on_cursor_enter')
+
+            elif action == 'windowleave':
+                self.dispatch('on_cursor_leave')
+
             elif action == 'joyaxismotion':
                 stickid, axisid, value = args
                 self.dispatch('on_joy_axis', stickid, axisid, value)
@@ -406,42 +640,35 @@ class WindowSDL(WindowBase):
             elif action in ('keydown', 'keyup'):
                 mod, key, scancode, kstr = args
 
-                key_swap = {
-                    SDLK_LEFT: 276, SDLK_RIGHT: 275, SDLK_UP: 273,
-                    SDLK_DOWN: 274, SDLK_HOME: 278, SDLK_END: 279,
-                    SDLK_PAGEDOWN: 281, SDLK_PAGEUP: 280, SDLK_SHIFTR: 303,
-                    SDLK_SHIFTL: 304, SDLK_SUPER: 309, SDLK_LCTRL: 305,
-                    SDLK_RCTRL: 306, SDLK_LALT: 308, SDLK_RALT: 307,
-                    SDLK_CAPS: 301, SDLK_INSERT: 277, SDLK_F1: 282,
-                    SDLK_F2: 283, SDLK_F3: 284, SDLK_F4: 285, SDLK_F5: 286,
-                    SDLK_F6: 287, SDLK_F7: 288, SDLK_F8: 289, SDLK_F9: 290,
-                    SDLK_F10: 291, SDLK_F11: 292, SDLK_F12: 293, SDLK_F13: 294,
-                    SDLK_F14: 295, SDLK_F15: 296, SDLK_KEYPADNUM: 300}
-
-                if platform == 'ios':
-                    # XXX ios keyboard suck, when backspace is hit, the delete
-                    # keycode is sent. fix it.
-                    key_swap[127] = 8  # back
-
                 try:
-                    key = key_swap[key]
+                    key = self.key_map[key]
                 except KeyError:
                     pass
 
                 if action == 'keydown':
                     self._update_modifiers(mod, key)
                 else:
-                    self._update_modifiers(mod)  # ignore the key, it
-                                                 # has been released
+                    # ignore the key, it has been released
+                    self._update_modifiers(mod)
 
                 # if mod in self._meta_keys:
                 if (key not in self._modifiers and
-                    key not in self.command_keys.keys()):
+                        key not in self.command_keys.keys()):
                     try:
-                        kstr = unichr(key)
+                        kstr_chr = unichr(key)
+                        try:
+                            # On android, there is no 'encoding' attribute.
+                            # On other platforms, if stdout is redirected,
+                            # 'encoding' may be None
+                            encoding = getattr(sys.stdout, 'encoding',
+                                               'utf8') or 'utf8'
+                            kstr_chr.encode(encoding)
+                            kstr = kstr_chr
+                        except UnicodeError:
+                            pass
                     except ValueError:
                         pass
-                #if 'shift' in self._modifiers and key\
+                # if 'shift' in self._modifiers and key\
                 #        not in self.command_keys.keys():
                 #    return
 
@@ -461,17 +688,10 @@ class WindowSDL(WindowBase):
             elif action == 'textinput':
                 text = args[0]
                 self.dispatch('on_textinput', text)
-                # XXX on IOS, keydown/up don't send unicode anymore.
-                # With latest sdl, the text is sent over textinput
-                # Right now, redo keydown/up, but we need to seperate both call
-                # too. (and adapt on_key_* API.)
-                #self.dispatch()
-                #self.dispatch('on_key_down', key, None, args[0],
-                #              self.modifiers)
-                #self.dispatch('on_keyboard', None, None, args[0],
-                #              self.modifiers)
-                #self.dispatch('on_key_up', key, None, args[0],
-                #              self.modifiers)
+
+            elif action == 'textedit':
+                text = args[0]
+                self.dispatch('on_textedit', text)
 
             # unhandled event !
             else:
@@ -480,10 +700,10 @@ class WindowSDL(WindowBase):
     def _do_resize(self, dt):
         Logger.debug('Window: Resize window to %s' % str(self.size))
         self._win.resize_window(*self._size)
-        self.dispatch('on_resize', *self.size)
+        self.dispatch('on_pre_resize', *self.size)
 
     def do_pause(self):
-        # should go to app pause mode.
+        # should go to app pause mode (desktop style)
         from kivy.app import App
         from kivy.base import stopTouchApp
         app = App.get_running_app()
@@ -508,7 +728,8 @@ class WindowSDL(WindowBase):
             action, args = event[0], event[1:]
             if action == 'quit':
                 EventLoop.quit = True
-                self.close()
+                break
+            elif action == 'app_willenterforeground':
                 break
             elif action == 'windowrestored':
                 break
@@ -518,7 +739,7 @@ class WindowSDL(WindowBase):
     def mainloop(self):
         # don't known why, but pygame required a resize event
         # for opengl, before mainloop... window reinit ?
-        #self.dispatch('on_resize', *self.size)
+        # self.dispatch('on_resize', *self.size)
 
         while not EventLoop.quit and EventLoop.status == 'started':
             try:
@@ -531,15 +752,10 @@ class WindowSDL(WindowBase):
                     raise
                 else:
                     pass
+        Logger.info("WindowSDL: exiting mainloop and closing.")
+        self.close()
 
-    #
-    # Pygame wrapper
-    #
     def _update_modifiers(self, mods=None, key=None):
-        # Available mod, from dir(pygame)
-        # 'KMOD_ALT', 'KMOD_CAPS', 'KMOD_CTRL', 'KMOD_LALT',
-        # 'KMOD_LCTRL', 'KMOD_LMETA', 'KMOD_LSHIFT', 'KMOD_META',
-        # 'KMOD_MODE', 'KMOD_NONE'
         if mods is None and key is None:
             return
         modifiers = set()
@@ -547,22 +763,30 @@ class WindowSDL(WindowBase):
         if mods is not None:
             if mods & (KMOD_RSHIFT | KMOD_LSHIFT):
                 modifiers.add('shift')
-            if mods & (KMOD_RALT | KMOD_LALT):
+            if mods & (KMOD_RALT | KMOD_LALT | KMOD_MODE):
                 modifiers.add('alt')
             if mods & (KMOD_RCTRL | KMOD_LCTRL):
                 modifiers.add('ctrl')
-            if mods & (KMOD_RMETA | KMOD_LMETA):
+            if mods & (KMOD_RGUI | KMOD_LGUI):
                 modifiers.add('meta')
+            if mods & KMOD_NUM:
+                modifiers.add('numlock')
+            if mods & KMOD_CAPS:
+                modifiers.add('capslock')
 
         if key is not None:
             if key in (KMOD_RSHIFT, KMOD_LSHIFT):
                 modifiers.add('shift')
-            if key in (KMOD_RALT, KMOD_LALT):
+            if key in (KMOD_RALT, KMOD_LALT, KMOD_MODE):
                 modifiers.add('alt')
             if key in (KMOD_RCTRL, KMOD_LCTRL):
                 modifiers.add('ctrl')
-            if key in (KMOD_RMETA, KMOD_LMETA):
+            if key in (KMOD_RGUI, KMOD_LGUI):
                 modifiers.add('meta')
+            if key == KMOD_NUM:
+                modifiers.add('numlock')
+            if key == KMOD_CAPS:
+                modifiers.add('capslock')
 
         self._modifiers = list(modifiers)
         return
@@ -570,7 +794,7 @@ class WindowSDL(WindowBase):
     def request_keyboard(self, callback, target, input_type='text'):
         self._sdl_keyboard = super(WindowSDL, self).\
             request_keyboard(callback, target, input_type)
-        self._win.show_keyboard()
+        self._win.show_keyboard(self._system_keyboard, self.softinput_mode)
         Clock.schedule_interval(self._check_keyboard_shown, 1 / 5.)
         return self._sdl_keyboard
 
@@ -586,3 +810,15 @@ class WindowSDL(WindowBase):
         if not self._win.is_keyboard_shown():
             self._sdl_keyboard.release()
 
+    def map_key(self, original_key, new_key):
+        self.key_map[original_key] = new_key
+
+    def unmap_key(self, key):
+        if key in self.key_map:
+            del self.key_map[key]
+
+    def grab_mouse(self):
+        self._win.grab_mouse(True)
+
+    def ungrab_mouse(self):
+        self._win.grab_mouse(False)
