@@ -2,6 +2,9 @@
 Tree View
 =========
 
+.. image:: images/treeview.png
+    :align: right
+
 .. versionadded:: 1.0.4
 
 
@@ -269,12 +272,14 @@ class TreeView(Widget):
         for key, value in self.root_options.items():
             setattr(tvlabel, key, value)
         self._root = self.add_node(tvlabel, None)
-        self.bind(
-            pos=self._trigger_layout,
-            size=self._trigger_layout,
-            indent_level=self._trigger_layout,
-            indent_start=self._trigger_layout)
-        self._trigger_layout()
+
+        trigger = self._trigger_layout
+        fbind = self.fbind
+        fbind('pos', trigger)
+        fbind('size', trigger)
+        fbind('indent_level', trigger)
+        fbind('indent_start', trigger)
+        trigger()
 
     def add_node(self, node, parent=None):
         '''Add a new node to the tree.
@@ -301,7 +306,7 @@ class TreeView(Widget):
             parent.nodes.append(node)
             node.parent_node = parent
             node.level = parent.level + 1
-        node.bind(size=self._trigger_layout)
+        node.fbind('size', self._trigger_layout)
         self._trigger_layout()
         return node
 
@@ -321,13 +326,15 @@ class TreeView(Widget):
                 'The node must be a subclass of TreeViewNode')
         parent = node.parent_node
         if parent is not None:
+            if node == self._selected_node:
+                node.is_selected = False
+                self._selected_node = None
             nodes = parent.nodes
             if node in nodes:
                 nodes.remove(node)
-                self._selected_node = None
             parent.is_leaf = not bool(len(nodes))
             node.parent_node = None
-            node.unbind(size=self._trigger_layout)
+            node.funbind('size', self._trigger_layout)
             self._trigger_layout()
 
     def on_node_expand(self, node):
@@ -346,6 +353,15 @@ class TreeView(Widget):
         node.is_selected = True
         self._selected_node = node
 
+    def deselect_node(self, *args):
+        '''Deselect any selected node.
+
+        .. versionadded:: 1.10.0
+        '''
+        if self._selected_node:
+            self._selected_node.is_selected = False
+            self._selected_node = None
+
     def toggle_node(self, node):
         '''Toggle the state of the node (open/collapsed).
         '''
@@ -441,12 +457,9 @@ class TreeView(Widget):
         self._do_layout_node(self.root, 0, self.top)
         # now iterate for calculating minimum size
         min_width = min_height = 0
-        count = 0
-        for node in self.iterate_open_nodes(self.root):
+        for count, node in enumerate(self.iterate_open_nodes(self.root)):
             node.odd = False if count % 2 else True
-            count += 1
-            min_width = max(min_width, node.width + self.indent_level +
-                            node.level * self.indent_level)
+            min_width = max(min_width, node.right - self.x)
             min_height += node.height
         self.minimum_size = (min_width, min_height)
 
@@ -508,7 +521,7 @@ class TreeView(Widget):
 
     .. versionadded:: 1.0.9
 
-    :attr:`minimum_width` is a :class:`kivy.properties.NumericProperty` and
+    :attr:`minimum_width` is a :class:`~kivy.properties.NumericProperty` and
     defaults to 0.
     '''
 
@@ -517,7 +530,7 @@ class TreeView(Widget):
 
     .. versionadded:: 1.0.9
 
-    :attr:`minimum_height` is a :class:`kivy.properties.NumericProperty` and
+    :attr:`minimum_height` is a :class:`~kivy.properties.NumericProperty` and
     defaults to 0.
     '''
 
